<?php

/**
 * Class GirdGallery_Galleries_Model_Galleries
 * Provides the logic to work with the galleries
 *
 * @package GirdGallery\Galleries\Model
 * @author Artur Kovalevsky
 */
class GirdGallery_Galleries_Model_Galleries extends GirdGallery_Core_BaseModel
{

    /**
     * @var string
     */
    protected $table;

    /**
     * Constructor
     * @param bool $debugEnabled
     */
    public function __construct($debugEnabled = false)
    {
        parent::__construct();

        $this->debugEnabled = (bool) $debugEnabled;
        $this->table = $this->db->prefix . 'gg_galleries';
    }

    /**
     * Returns the data of the gallery by the identifier
     *
     * @param int $id The identifier of the gallery
     * @return object|null
     */
    public function getById($id)
    {
        $query = $this->getQueryBuilder()->select('*')
            ->from($this->table)
            ->where('id', '=', (int) $id);

        if ($gallery = $this->db->get_row($query->build())) {
            $gallery = $this->extend($gallery);
        }

        return $gallery;
    }

    /**
     * Returns the array of the galleries
     *
     * @return array|null
     */
    public function getAll()
    {
        if ($galleries = $this->getList()) {
            foreach ($galleries as $index => $gallery) {
                $galleries[$index] = $this->extend($gallery);
            }
        }

        return $galleries;
    }

    /**
     * Returns the array of the NOT extended galleries
     *
     * @return null|array
     */
    public function getList()
    {
        $query = $this->getQueryBuilder()->select('*')
            ->from($this->table);

        return $this->db->get_results($query->build());
    }

    /**
     * Adds the empty gallery to the database
     * @param string $title
     * @return bool TRUE on success, FALSE otherwise
     */
    public function add($title)
    {
        $query = $this->getQueryBuilder()->insertInto($this->table)
            ->fields('title')
            ->values($title);

        if (!$this->db->query($query->build())) {
            return false;
        }

        $this->insertId = $this->db->insert_id;

        do_action(apply_filters('gg_hooks_prefix', 'gallery_created'), $this->db->insert_id);

        return true;
    }

    /**
     * Creates the new gallery from the HTTP request
     *
     * @param Rsc_Http_Request $request The HTTP request
     * @param Rsc_Lang $lang The instance of the language class
     * @return bool TRUE on success, FALSE otherwise
     * @throws GirdGallery_Galleries_Exception_EmptyTitleException
     */
    public function createFromRequest(Rsc_Http_Request $request, Rsc_Lang $lang)
    {
        if (!$title = $request->post->get('title')) {
            $title = $lang->translate('Unnamed gallery');
        }

        $title = htmlspecialchars($request->post->get('title'));
        return $this->add($title);
    }

    /**
     * Renames the gallery by the specified identifier
     *
     * @param int $galleryId The identifier of the gallery
     * @param string $title The new title of the gallery
     * @return bool TRUE on success, FALSE otherwise
     */
    public function rename($galleryId, $title)
    {
        $query = $this->getQueryBuilder()->update($this->table)
            ->fields('title')
            ->values(htmlspecialchars($title, ENT_QUOTES, get_bloginfo('charset')))
            ->where('id', '=', (int) $galleryId);

        if (!$this->db->query($query->build())) {

            if ($this->logger) {
                $this->logger->error('Failed to execute query: ({query}) in method {method}', array(
                    'query'  => $query,
                    'method' => __METHOD__,
                ));
            }

            return false;
        }

        return true;
    }

    /**
     * Renames the gallery from the HTTP request
     *
     * @param Rsc_Http_Request $request An instance of the HTTP request
     * @param Rsc_Lang $lang An instance of the translation class
     * @throws RuntimeException When rename query return FALSE
     * @throws InvalidArgumentException If at least one argument is invalid
     */
    public function renameFromRequest(Rsc_Http_Request $request, Rsc_Lang $lang)
    {
        $galleryId = $request->post->get('gallery_id');
        $title     = $request->post->get('title');

        if (!is_numeric($galleryId)) {
            throw new InvalidArgumentException($lang->translate('The identifier of the gallery is invalid'));
        }

        if (empty($title)) {
            throw new InvalidArgumentException($lang->translate('Title is empty'));
        }

        if (!$this->rename($galleryId, $title)) {
            throw new RuntimeException(
                $lang->translate('Failed to rename the gallery')
            );
        }
    }

    /**
     * Deletes gallery from the database from the HTTP request
     *
     * @param Rsc_Http_Request $request An instance of the HTTP request
     * @param Rsc_Lang $lang An instance of the translation class
     * @param string $hooksPrefix The prefix for the plugin hooks
     * @throws RuntimeException
     * @throws UnexpectedValueException
     */
    public function deleteFromRequest(Rsc_Http_Request $request, Rsc_Lang $lang, $hooksPrefix = null)
    {
        $galleryId = $request->query->get('gallery_id');

        if (!is_numeric($galleryId)) {
            throw new UnexpectedValueException($lang->translate('Invalid gallery identifier specified'));
        }

        if (!$this->delete($galleryId, $hooksPrefix)) {
            throw new RuntimeException($lang->translate('Failed to delete the gallery'));
        }
    }

    /**
     * Deletes the gallery by the identifier
     *
     * @param int $galleryId The identifier of the gallery
     * @param string $hooksPrefix The prefix for the plugin hooks
     * @return bool TRUE on success, FALSE otherwise
     */
    public function delete($galleryId, $hooksPrefix = null)
    {
        $query = $this->getQueryBuilder()->deleteFrom($this->table)
            ->where('id', '=', (int) $galleryId);

        if (!$this->db->query($query->build())) {

            if ($this->logger) {
                $this->logger->error('Failed to execute query ({query}) in method {method}', array(
                    'query'  => $query,
                    'method' => __METHOD__,
                ));
            }

            return false;
        }

        do_action($hooksPrefix . 'gallery_delete', $galleryId);

        return true;
    }

    /**
     * Sets the settings for the gallery
     *
     * @param int $galleryId
     * @param int $settingsId
     * @return bool TRUE on success, FALSE otherwise
     */
    public function setSettings($galleryId, $settingsId)
    {
        $query = $this->getQueryBuilder()->update($this->table)
            ->fields('settings_id')
            ->values((int) $settingsId)
            ->where('id', '=', (int) $galleryId);

        if (!$this->db->query($query->build())) {
            return false;
        }

        return true;
    }

    /**
     * Sets the default settings for the gallery
     *
     * @param int $galleryId
     * @return bool TRUE on success, FALSE otherwise
     */
    public function setDefaultSettings($galleryId)
    {
//        $settings = new GirdGallery_Galleries_Model_Settings();
//        $settings->setDebugEnabled($this->debugEnabled);
//
//        if ($logger = $this->logger) {
//            $settings->setLogger($logger);
//        }
//
//        if (!$defaults = $settings->getDefaults()) {
//            if ($logger) {
//                $logger->error('Failed to load default gallery settings');
//            }
//
//            return false;
//        }
//
//        if (!$settings->save($defaults)) {
//            if ($logger) {
//                $logger->error('Failed to save defaults for the gallery {gid}', array(
//                    'gid' => $galleryId,
//                ));
//            }
//
//            return false;
//        }
//
//        if (!$settingsId = $settings->getInsertId()) {
//            if ($logger) {
//                $logger->error('Failed get the settings identifier');
//            }
//
//            return false;
//        }
//
//        if (!$this->setSettings($galleryId, $settingsId)) {
//            if ($logger) {
//                $logger->error('Failed to save settings {sid} for the gallery {gid}', array(
//                    'sid' => $settingsId,
//                    'gid' => $galleryId,
//                ));
//            }
//
//            return false;
//        }
//
//        if ($logger) {
//            $logger->debug('The settings {sid} is successfully attached to the gallery {gid}', array(
//                'sid' => $settingsId,
//                'gid' => $galleryId,
//            ));
//        }

        return true;
    }

    /**
     * Extends default gallery select
     *
     * @param object $gallery The result of the SELECT
     * @return mixed
     */
    protected function extend($gallery)
    {
        if (!is_object($gallery)) {
            if ($this->debugEnabled) {
                wp_die (sprintf('The $gallery variable must be type of object, %s given', gettype($gallery)));
            }

            return $gallery;
        }

        $models = array();
        $resources = new GirdGallery_Galleries_Model_Resources($this->debugEnabled);
        $resourcesData = $resources->getByGalleryId($gallery->id);

        if (!$resourcesData) {
            return $gallery;
        }

        $gallery->photos = array();
        $gallery->settings = array();

        $models['photos'] = new GirdGallery_Photos_Model_Photos($this->debugEnabled);
        $models['folders'] = new GirdGallery_Photos_Model_Folders($this->debugEnabled);
        $models['settings'] = new GirdGallery_Galleries_Model_Settings($this->debugEnabled);

        foreach ($resourcesData as $data) {
            switch ($data->resource_type) {
                case 'folder':
                    if ($photos = $models['folders']->getPhotosById($data->resource_id)) {
                        foreach ($photos as $photo) {
                            if (!$this->isExcluded($gallery->id, $data->resource_id, $photo->id)) {
                                $gallery->photos[] = $photo;
                            }
                        }
                    }
                    break;

                case 'photo':
                    $gallery->photos[] = $models['photos']->getById($data->resource_id);
                    break;
            }
        }

//        $gallery->settings = $models['settings']->getById($gallery->settings_id);

        return $gallery;
    }

    public function isExcluded($galleryId, $folderId, $photoId)
    {
        $table = $this->db->prefix . 'gg_galleries_excluded';

        $query = $this->getQueryBuilder()->select('*')
            ->from($table)
            ->where('gallery_id', '=', (int) $galleryId)
            ->andWhere('folder_id', '=', (int) $folderId)
            ->andWhere('photo_id', '=', (int) $photoId);

        return (null !== $this->db->get_row($query->build()));
    }

    /**
     * Sets a logger instance on the object
     *
     * @param Rsc_Logger_Interface $logger
     * @return null
     */
    public function setLogger(Rsc_Logger_Interface $logger)
    {
        $this->logger = $logger;
    }
}