<?php

/**
 * Class Rsc_Environment
 * Framework Environment
 *
 * @package Rsc
 * @author Artur Kovalevsky
 * @copyright Copyright (c) 2014, ReadyShoppingCart
 * @link http://readyshoppingcart.com/
 */
class Rsc_Environment
{

    const ENV_DEVELOPMENT = 'development';
    const ENV_PRODUCTION  = 'production';

    /**
     * @var array
     */
    protected $defaults = array(
        'environment' => self::ENV_PRODUCTION,
    );

    /**
     * @var string
     */
    protected $pluginName;

    /**
     * @var Rsc_Config
     */
    protected $config;

    /**
     * @var Rsc_Cache
     */
    protected $cache;

    /**
     * @var Rsc_ClassLoader
     */
    protected $loader;

    /**
     * @var Rsc_Lang
     */
    protected $lang;

    /**
     * @var Rsc_Menu_Page
     */
    protected $menu;

    /**
     * @var Rsc_Resolver
     */
    protected $resolver;

    /**
     * @var Twig_Environment
     */
    protected $twig;

    /**
     * @var Rsc_Logger_Interface
     */
    protected $logger;

    /**
     * @var bool
     */
    protected $profilerEnabled;

    /**
     * Constructor
     * @param string $pluginName Plugin name
     * @param null $version
     */
    public function __construct($pluginName, $version = null)
    {
        $this->pluginName = $pluginName;

        /* Class loader */
        $this->loader = new Rsc_ClassLoader();

        /* Modules resolver */
        $this->resolver = new Rsc_Resolver($this);

        /* Config */
        $this->config = new Rsc_Config($this->defaults);
        $this->config->add('plugin_name', $this->pluginName);
        $this->config->add('plugin_version', $version);
    }

    /**
     * Configure plugin environment
     * @param array $parameters An associative array with the parameters
     * @return Rsc_Environment
     */
    public function configure(array $parameters)
    {
        $this->config->merge($parameters);
        return $this;
    }

    /**
     * Run plugin
     */
    public function run()
    {
        $this->loader->add('BarsMaster', dirname(dirname(__FILE__)));
        $this->loader->add('Twig', dirname(dirname(__FILE__)));

        if ($this->config->has('plugin_prefix') && $this->config->has('plugin_source')) {
            $prefix = $this->config->get('plugin_prefix');
            $path = $this->config->get('plugin_source');

            $this->loader->add($prefix, $path);
        }

        $this->loader->register();

        /* Twig */
        $this->twig = new Twig_Environment(new Twig_Loader_Filesystem(dirname(dirname(dirname(__FILE__))) . '/app/templates'), array(
            'cache' => $this->config->get('plugin_cache_twig', false),
            'debug' => $this->config->isEnvironment(self::ENV_DEVELOPMENT),
        ));

        /** @TODO TRY CATCH */
        $this->getLang()->loadTextDomain();

        /** @TODO THROW TRY CATCH */
        if ($this->config->has('plugin_menu')) {
            $this->menu = new Rsc_Menu_Page($this->resolver);

            foreach ($parameters = $this->config->get('plugin_menu') as $key => $value) {
                if (method_exists($this->menu, $method = sprintf('set%s', str_replace('_', '', $key)))) {
                    call_user_func_array(array($this->menu, $method), array($value));
                }
            }

            $this->menu->register();
        }

        $this->twig->addGlobal('environment', $this);
        $this->twig->addGlobal('request', new Rsc_Http_Request());

        $this->resolver->init();
    }

    /**
     * Returns an instance of the Twig
     * @return Twig_Environment
     */
    public function getTwig()
    {
        return $this->twig;
    }

    /**
     * Returns an instance of the language class
     * @return Rsc_Lang
     */
    public function getLang()
    {
        if ($this->lang === null) {

            if (!$domain = $this->config->get('lang_domain')) {
                $domain = preg_replace("/[^a-z0-9_-]/", '', strtolower($this->pluginName));
                $this->config->add('lang_domain', $domain);
            }

            if (!$path = $this->config->get('lang_path')) {
                $path = dirname(dirname(dirname(__FILE__))) . '/app/langs';

                $this->config->add('lang_path', $path);
            }

            $this->lang = new Rsc_Lang($domain, $path, $this->cache);
        }

        return $this->lang;
    }

    /**
     * Returns plugin environment configurations
     * @return Rsc_Config
     */
    public function getConfig()
    {
        return $this->config;
    }

    /**
     * Returns an instance of the caching class
     * @return Rsc_Cache
     */
    public function getCache()
    {
        return $this->cache;
    }

    /**
     * Returns ClassLoader
     * @return Rsc_ClassLoader
     */
    public function getLoader()
    {
        return $this->loader;
    }

    /**
     * Returns an instance of the current menu page
     * @return \Rsc_Menu_Page
     */
    public function getMenu()
    {
        return $this->menu;
    }

    /**
     * Returns an instance of the modules resolver
     * @return Rsc_Resolver
     */
    public function getResolver()
    {
        return $this->resolver;
    }

    /**
     * @param \Rsc_Logger_Interface $logger
     * @return Rsc_Environment
     */
    public function setLogger(Rsc_Logger_Interface $logger)
    {
        $this->logger = $logger;
        return $this;
    }

    /**
     * @return \Rsc_Logger_Interface
     */
    public function getLogger()
    {
        return $this->logger;
    }


    /**
     * Returns the URL to the plugin page
     * @return null|string
     */
    public function getUrl()
    {
        if ($this->menu === null) {
            return null;
        }

        return admin_url(sprintf('admin.php?page=%s', $this->menu->getMenuSlug()));
    }

    /**
     * @param string $module The name of the module
     * @param string $action The name of the action (default indexAction)
     * @param array $parameters An associative array of the parameters
     * @return string
     */
    public function generateUrl($module, $action = 'index', array $parameters = array())
    {
        $url = $this->getUrl() . '&module=' . strtolower($module);

        if (!empty($action) && $action != 'index') {
            $url .= '&action=' . $action;
        }

        if (!empty($parameters)) {
            $url .= '&' . http_build_query($parameters);
        }

        return $url;
    }

    /**
     * Returns an instance of the specified module
     * @param string $module The name of the module
     * @return Rsc_Mvc_Module|null
     */
    public function getModule($module)
    {
        return $this->resolver->getModules()->get(strtolower($module));
    }

    /**
     * Translates specified text
     * @param string $msgid The text to translate
     * @return string|void
     */
    public function translate($msgid)
    {
        return $this->getLang()->translate($msgid);
    }

    /**
     * Set an instance of the caching class
     * @param Rsc_Cache_Interface $adapter
     * @return Rsc_Environment
     */
    public function setCacheAdapter(Rsc_Cache_Interface $adapter)
    {
        $this->cache = new Rsc_Cache($adapter);
        return $this;
    }

    /**
     * Set profiler state
     * @param boolean $profilerEnabled
     * @return Rsc_Environment
     */
    public function setProfilerEnabled($profilerEnabled)
    {
        $this->profilerEnabled = (bool)$profilerEnabled;
        return $this;
    }

    /**
     * Get profiler state
     * @return boolean
     */
    public function getProfilerEnabled()
    {
        return $this->profilerEnabled;
    }

    /**
     * Checks whether the current instance of the environment equals to the
     * specified.
     * @param string $environment
     * @return bool
     */
    public function is($environment)
    {
        return $this->config->isEnvironment($environment);
    }

    public function isProd()
    {
        return $this->is(self::ENV_PRODUCTION);
    }

    public function isDev()
    {
        return $this->is(self::ENV_DEVELOPMENT);
    }
} 