<?php

/**
 * Class Rsc_Mvc_Module
 * Base module
 *
 * @package Rsc\Mvc
 * @author Artur Kovalevsky
 * @copyright Copyright (c) 2014, ReadyShoppingCart
 * @link http://readyshoppingcart.com/
 */
class Rsc_Mvc_Module
{

    /**
     * @var Rsc_Environment
     */
    private $environment;

    /**
     * @var string
     */
    private $location;

    /**
     * @var Rsc_Http_Request
     */
    private $request;

    /**
     * @var Rsc_Mvc_Controller
     */
    private $controller;

    /**
     * Constructor
     * @param Rsc_Environment $environment An instance of the current environment
     * @param string $location Full path to the directory where is module
     */
    public function __construct(Rsc_Environment $environment, $location)
    {
        $this->environment = $environment;
        $this->location = $location;
    }

    /**
     * @param string $method The name of the method
     * @param array $arguments An array of arguments
     * @return mixed
     * @throws BadMethodCallException If specified method does not exists
     */
    public function __call($method, $arguments)
    {
        if (!method_exists($this->environment, $method)) {
            throw new BadMethodCallException(sprintf('Unexpected method: %s', $method));
        }

        return call_user_func_array(array($this->environment, $method), $arguments);
    }

    /**
     * Returns an instance of the current environment
     * @return Rsc_Environment
     */
    public function getEnvironment()
    {
        return $this->environment;
    }

    /**
     * Returns full path to the directory where is module
     * @return string
     */
    public function getLocation()
    {
        return str_replace('\\', '/', $this->location);
    }

    /**
     * Returns the url to the current module's page or NULL if the menu page is not configured
     * @return null|string
     */
    public function getUrl()
    {
        if (null !== $url = $this->environment->getUrl()) {
            return $url . '&module=' . end(explode('_', strtolower(basename($this->location))));
        }

        return null;
    }

    /**
     * Returns the URL to the module location
     * @return string
     */
    public function getLocationUrl()
    {
        $path = str_replace('\\','/', WP_CONTENT_DIR . '/plugins');

        return plugins_url(str_replace($path, '', $this->location));
    }

    /**
     * Returns the name of the current module
     * @return string
     */
    public function getModuleName()
    {
        return strtolower(basename($this->location));
    }

    /**
     * Returns an instance of controller
     * @return null|Rsc_Mvc_Controller
     */
    public function getController()
    {
        if ($this->controller === null) {
            if (!is_file($this->location . '/Controller.php')) {
                return null;
            }

            $prefix = $this->environment->getConfig()->get('plugin_prefix');
            if (!class_exists($classname = $prefix . '_' . ucfirst($this->getModuleName()) . '_Controller', false)) {
                $this->controller = new $classname($this->environment, $this->request);
            }
        }

        return $this->controller;
    }

    /**
     * Returns the HTTP request to the module
     * @return Rsc_Http_Request
     */
    public function getRequest()
    {
        return $this->request;
    }

    /**
     * Set HTTP request to the module
     * @param Rsc_Http_Request $request HTTP request
     * @return Rsc_Mvc_Module
     */
    public function setRequest(Rsc_Http_Request $request)
    {
        $this->request = $request;
        return $this;
    }

    public function handle()
    {
        $action = $this->request->query->get('action', 'index') . 'Action';

        if (method_exists($this->getController(), $action)) {
            return call_user_func_array(array($this->getController(), $action), array(
                $this->request
            ));
        }

        $twig = $this->environment->getTwig();

        return Rsc_Http_Response::create()->setContent($twig->render('404.twig'));
    }

    /**
     * Triggered when the resolver doing initialization of module
     */
    public function onInit()
    {

    }

    /**
     * Triggered on plugin activation
     */
    public function onInstall()
    {

    }

    /**
     * Triggered on plugin uninstall
     */
    public function onUninstall()
    {

    }

    /**
     * Triggered on plugin deactivation
     */
    public function onDeactivation()
    {

    }
}