<?php

/**
 * Class Rsc_Resolver
 * Modules resolver
 *
 * @package Rsc
 * @author Artur Kovalevsky
 * @copyright Copyright (c) 2014, ReadyShoppingCart
 * @link http://readyshoppingcart.com/
 */
class Rsc_Resolver
{

    /**
     * @var Rsc_Environment
     */
    private $environment;

    /**
     * @var Rsc_Common_Collection
     */
    protected $modules;

    /**
     * @var Rsc_Http_Request
     */
    protected $request;

    /**
     * @var array
     */
    protected $routes;

    /**
     * Constructor
     * @param Rsc_Environment $environment An instance of current environment
     */
    public function __construct(Rsc_Environment $environment)
    {
        $this->environment = $environment;
        $this->modules = new Rsc_Common_Collection();
        $this->request = new Rsc_Http_Request();
    }

    /**
     * Returns an instance of environment
     * @return Rsc_Environment
     */
    public function getEnvironment()
    {
        return $this->environment;
    }

    /**
     * Returns the collection of the found modules
     * @return Rsc_Common_Collection
     */
    public function getModules()
    {
        return $this->modules;
    }

    public function setRoute($slug, $module)
    {
        $this->routes[$slug] = $module;
        return $this;
    }

    /**
     * Retrieves modules and doing initialization
     */
    public function init()
    {
        $config = $this->environment->getConfig();
        $hasHooksPrefix = $config->has('hooks_prefix');

        if (null === $modules = $this->getModulesFromCache()) {
            $modules = $this->getModulesList();
            $this->setModulesCache($modules);
        }

        /** @var Rsc_Mvc_Module $instance */
        foreach ($modules as $name => $instance) {

            /* Set http request to the module */
            $instance->setRequest($this->request)->onInit();

            /* Register hooks */
            $index = dirname(dirname(dirname(__FILE__))) . '/index.php';
            register_activation_hook($index, array($instance, 'onInstall'));
            register_deactivation_hook($index, array($instance, 'onDeactivation'));
            register_uninstall_hook($index, array($instance, 'onUninstall'));

            /* We add module's location to the template & config loaders */

            /** @var Twig_Loader_Filesystem $loader */
            try {
                $loader = $this->environment->getTwig()->getLoader();
                $loader->addPath($instance->getLocation() . '/views', strtolower($name));
            } catch (Twig_Error_Loader $e) {  }

            $config->getLoader()->add($instance->getLocation() . '/configs', strtolower($name));

            /* Add module to the resolver */
            $this->modules->add(strtolower($name), $instance);

            if ($hasHooksPrefix) {
                do_action(
                    $config->get('hooks_prefix') . 'after_' . strtolower($name) . '_loaded',
                    $this->modules[strtolower($name)]
                );
            }
        }

        if ($hasHooksPrefix) {
            do_action($config->get('hooks_prefix') . 'after_modules_loaded');
        }
    }

    /**
     * Parses current routes and loads the module that matched for that route
     */
    public function resolve()
    {
        $query = $this->request->query;

        if (in_array($query->get('page'), array_keys($this->routes))) {
            $name = $this->routes[$query->get('page')];
        } else {
            /** @var Rsc_Mvc_Module $module */
            $name = $query->get('module', $this->environment->getConfig()->get('default_module'));
        }

        if (!$this->modules->has($name)) {
            wp_die ('The module that you requested is not found');
        }

        /** @var Rsc_Http_Response $response */
        $response = $module = $this->modules->get(strtolower($name))->setRequest($this->request)->handle();

        if (!is_object($response) || !$response instanceof Rsc_Http_Response) {
            wp_die ('The controller must return the response');
        }

        echo $response->getContent();
    }

    /**
     * Returns an array of modules from the cache if it exists
     * @return array|null
     */
    protected function getModulesFromCache()
    {
//        if (null !== $cache = $this->environment->getCache()) {
//            if (null !== $modules = $cache->get('modules')) {
//                return $modules;
//            }
//        }

        return null;
    }

    /**
     * Saving an array of modules to the cache
     * @param array $modules
     */
    protected function setModulesCache(array $modules)
    {
        if (null !== $cache = $this->environment->getCache()) {
//            $cache->set('modules', $modules, 31556926);
        }
    }

    /**
     * Returns an array of found modules or NULL if at least one module did not found
     * @return array|null
     */
    protected function getModulesList()
    {
        $modules = array();
        $nodes = array();
        $prefix = $this->environment->getConfig()->get('plugin_prefix');

        if (null !== $src = $this->environment->getConfig()->get('plugin_source')) {
            $nodes = glob(trailingslashit($src) . $prefix . '/*');

            if (empty($nodes) || $nodes === false) {
                return null;
            }
        }

        foreach ($nodes as $node) {

            $node = str_replace('\\', '/', $node);

            if (is_dir($node) && file_exists($module = $node . '/Module.php')) {
                $classname = $prefix . '_' . basename($node) . '_Module';
                $name = strtolower(basename(dirname($module)));

                $modules[$name] = new $classname($this->environment, $node);
            }
        }

        return $modules;
    }
} 